#
# AutoDemo.py
#
# This is an automatic version of the Demo.py script - it just loops forever
# pressing the buttons for you!.
#
# See the associated documentation for details of the resources required to run
# Demo.py and any associated installation tasks.
#
# Double-clicking this script, or invoking it in a command window with the
# command    python Demo.py    , will cause it to use the machines described
# by HOST1/DESCR1 and HOST2/DESCR2 as the two ends of the TRANSFER commands.
# The machine running this script must have direct access to HOST1 and HOST2.
#
# Alternatively, if HOST3 is a machine with direct access to HOST1 and HOST2 and
# the machine running this script has access to HOST3 then starting the script
# with the command   python Demo.py host3-address "host3 description"    will
# cause HOST3 to relay the scripts commands to HOST1 and HOST2 and relay their
# results back to the script.
#
# NOTE: an appropriate version of ADI-TTCP must already be running and awaiting
# a control stream connection on HOST1, HOST2 and HOST3 (if used) BEFORE this
# script starts running.
#

import sys, time
import ttcpcontrol  as ttcp
import Tkinter      as Tk
import tkFont       as Tf
import tktable      as Tt
import ScrolledText as St

# the test machines
#    HOST1 and HOST2 perform the actual data transfers while HOST3 is
#    an optional host for relaying control stream commands to the other two.
HOST1  = "192.168.0.100"
DESCR1 = "BF537 rev 0.2 600/120 MHz  All caches on"

HOST2  = "192.168.0.101"
DESCR2 = "BF537 rev 0.2 600/120 MHz  All caches on"

# RELAY not used by default; may be specified on script's command line
HOST3  = None
DESCR3 = "Unspecified platform"

ctrl1 = None
ctrl2 = None
ctrl3 = None

# identifiers of various after() callback requests
blink     = None        # for pulsing the RUN TEST button when test running

# initial colour of RUN TEST button when test starts to run
rbcolour  = 'pink'

################################################################################

def program_exit():
    """Called when Control and Results window is deleted by user. Cancels any
    outstanding callbacks, closes the control streams, destroys the top level
    window and exits.
    """
    global blink

    if blink != None: root.after_cancel(blink)

    if ctrl1: ctrl1.close()
    if ctrl2: ctrl2.close()
    if ctrl3: ctrl3.close()

    ttcp.CtrlStream.fini()
    root.destroy()

    sys.exit("Quit")

def create_table(master, caption, row, col):
    """Create a tktable and associated array variable and return them as a
    tuple. master is the containing frame and row and col are the coordinates
    within master's grid for the table.
    """

    # create Tk array to hold table values (including headings)
    arr = Tt.ArrayVar(master)
    # insert headings
    arr.set('0,0', '#')
    arr.set('0,1', 'Buf Size')
    if (col & 1) == 0:
        arr.set('0,2', 'Reqd MB/s')     # for transmit table
    else:
        arr.set('0,2', 'Bytes Rcvd')    # for receive table (meaningful for UDP)
    arr.set('0,3', 'Actual MB/s')
    arr.set('0,4', 'CPU Use')

    # create a caption and a tktable to display the array contents
    f = Tk.Frame(master, padx=3, pady=3, relief='groove', borderwidth=2)
    l = Tk.Label(f,
                 text=caption,
                 foreground='dark blue',
                 font=tabcap,
                 justify='center')

    l.pack(side='top', fill='x')

    tab = Tt.Table(f,
                   font=tabfont,
                   ipadx=3,
                   ipady=3,
                   borderwidth=1,
                   relief='groove',
                   background='white',
                   foreground='black',
                   rows=14,
                   cols=5,
                   state='disabled',
                   titlerows=1,
                   titlecols=0,
                   variable=arr)
    tab.tag_configure('active', background='LightBlue2')
    tab.tag_configure('celltag', background='LightBlue2')
    tab.tag_configure('title',
                      font=tabbold,
                      background='white',
                      foreground='black')
    tab.width(0, 4)
    tab.width(1, 8)
    tab.width(2, 11)
    tab.width(3, 11)
    tab.width(4, 8)

    tab.pack(side='bottom', fill='both')

    f.grid(column=col, row=row, sticky='news')

    return (tab, arr)

def cleartables():
    # clear the result tables
    tab1.configure(state='normal')
    tab2.configure(state='normal')
    tab3.configure(state='normal')
    tab4.configure(state='normal')
    for r in range(1,14):
        for c in xrange(5):
            index = '%i,%i' % (r,c)
            tab1.set(index, '')
            tab1.tag_cell('', index)
            tab2.set(index, '')
            tab2.tag_cell('', index)
            tab3.set(index, '')
            tab3.tag_cell('', index)
            tab4.set(index, '')
            tab4.tag_cell('', index)
    tab1.configure(state='disabled')
    tab2.configure(state='disabled')
    tab3.configure(state='disabled')
    tab4.configure(state='disabled')

def readtxt(txt):
    """Search the supplied string (the result of the transmit or receive side
    of a single TRANSFER) for specific items. Return a tuple of the buffer
    length, required and actual data rates and CPU usage as strings and the
    total number of bytes sent or received as a real.

    This code is not very robust and will raise exceptions if it doesn't see
    what it expects.
    """
    seek = 'Buffer Size : '
    x = txt.index(seek)
    x += len(seek)
    t = txt[x:]
    x = t.index(';')
    buflen = '%6s' % t[:x]

    reqd = ''
    seek = 'Data Rate   : '
    x = txt.find(seek)
    if x > -1:
        x += len(seek)
        t = txt[x:]
        x = t.index(' ')
        f = float(t[:x]) / 1000000.0
        reqd = '%5.2f' % f

    seek = 'real seconds = '
    x = txt.index(seek)
    x += len(seek)
    t = txt[x:]
    x = t.index(' ')
    f = float(t[:x])
    act = '%5.2f' % f

    seek = 'during transfer: '
    x = txt.index(seek)
    x += len(seek)
    t = txt[x:]
    x = t.find('%')
    if x > -1:
        f = float(t[:x])
        cpu = '%5.2f%%' % f
    else:
        cpu = ' N/A'

    seek = ' bytes in'
    x = txt.index(seek)
    t = txt[:x]
    x = t.rindex('\n')
    bytes = float(t[x+1:])

    return (buflen, reqd, act, cpu, bytes)


def filltablerow(tfr, ttab, rtab, row, x, selcol):
    """Write the transmit and receive information for TRANSFER 'tfr' into the
    supplied tables. 'x' is the information for column 0 and 'selcol' is the
    column which should be highlighted (set to -1 if none).
    """
    b, r, a, u, tbytes = readtxt(tfr.xmtTxt)
    vals = (x, b, r, a, u)
    ttab.configure(state='normal')
    for c in xrange(5):
        index = '%i,%i' % (row, c)
        ttab.set(index, vals[c])
        if c == selcol:
            ttab.tag_cell('celltag', index)
    ttab.configure(state='disabled')
    b, r, a, u, rbytes = readtxt(tfr.rcvTxt)
    # calculate received bytes as percentage of transmitted (for UDP)
    r = '%5.2f%%' % (rbytes/tbytes * 100)
    vals = (x, b, r, a, u)
    rtab.configure(state='normal')
    for c in xrange(5):
        index = '%i,%i' % (row, c)
        rtab.set(index, vals[c])
        if c == selcol:
            rtab.tag_cell('celltag', index)
    rtab.configure(state='disabled')


def test0():
    cleartables()

    # TCP transfers with various buffer sizes
    r = 1
    for blen in [1400, 1200, 1024, 512]:
        ttcp.Transfer(ctrl1,
                      ctrl2,
                      port=7010,
                      period=15,
                      buflen=blen,
                      format='M')
        res = ttcp.Transfer.Run()
        filltablerow(res[0], tab1, tab2, r, '', 1)
        r += 1

    # UDP transfers with various buffer sizes
    r = 1
    for blen in [1400, 1200, 1024, 512]:
        ttcp.Transfer(ctrl1,
                      ctrl2,
                      port=7010,
                      proto=ttcp.UDP,
                      period=15,
                      buflen=blen,
                      rate=11910000,
                      format='M')
        res = ttcp.Transfer.Run()
        filltablerow(res[0], tab3, tab4, r, '', 1)
        r += 1

def test1():
    cleartables()

    # TCP transfers at various data rates
    r = 1
    for rate in [500000, 2000000, 5000000, 10000000]:
        ttcp.Transfer(ctrl1,
                      ctrl2,
                      port=7010,
                      period=15,
                      buflen=1400,
                      rate=rate,
                      format='M')
        res = ttcp.Transfer.Run()
        filltablerow(res[0], tab1, tab2, r, '', 3)
        r += 1

    # UDP transfers at various data rates
    r = 1
    for rate in [500000, 2000000, 5000000, 10000000]:
        ttcp.Transfer(ctrl1,
                      ctrl2,
                      port=7010,
                      proto=ttcp.UDP,
                      period=15,
                      buflen=1400,
                      rate=rate,
                      format='M')
        res = ttcp.Transfer.Run()
        filltablerow(res[0], tab3, tab4, r, '', 3)
        r += 1

def test2():
    cleartables()

    # various numbers of TCP transfers at fixed throughput
    r = 1
    for num in [1, 4, 8]:
        for x in xrange(num):
            ttcp.Transfer(ctrl1,
                          ctrl2,
                          port=7000 + x,
                          period=15,
                          buflen=1400,
                          rate=8000000/num,
                          format='M')
        res = ttcp.Transfer.Run()
        n = '%3i' % num
        for y in res:
            filltablerow(y, tab1, tab2, r, n, 0)
            r += 1

    # various numbers of UDP transfers at fixed throughput
    r = 1
    for num in [1, 4, 8]:
        for x in xrange(num):
            ttcp.Transfer(ctrl1,
                          ctrl2,
                          port=7000 + x,
                          proto=ttcp.UDP,
                          period=15,
                          buflen=1400,
                          rate=8000000/num,
                          format='M')
        res = ttcp.Transfer.Run()
        n = '%3i' % num
        for y in res:
            filltablerow(y, tab3, tab4, r, n, 0)
            r += 1

tests = (test0, test1, test2)

# headings and descriptions for each of the tests
testdescrs = (
  ('Test peak throughput versus buffer size.',
'A series of single TCP and UDP transfers is performed between the two machines. ' +
'Each transfer uses a different buffer size for writing to and reading from ' +
'the socket. The buffer sizes used are 1400, 1200, 1024, and 512 bytes.\n\n' +
'Each transfer runs for 15 seconds so the whole test should take ' +
'just over two minutes.'
  ),
  ('Test CPU use versus data rate.',
'A series of single TCP and UDP transfers is performed at different fixed data ' +
'rates to see the effect on CPU usage. The data rates used are 0.5, 2.0, ' +
'5.0 and 10.0 MB/sec.\n\n' +
'Each transfer runs for 15 seconds so the whole test should take ' +
'just over two minutes.'
  ),
  ('Test CPU use versus number of streams.',
'Three sets of concurrent TCP or UDP transfers are run at a fixed throughput of ' +
'8.0 MB/sec. Each set runs a different number of concurrent transfers: ' +
'1 at 8MB/s, 4 at 2MB/s or 8 at 1MB/s.\n\n' +
'Each transfer runs for 15 seconds so the whole test should take ' +
'less than two minutes.'
  )
)

def radiocheck():
    """Callback for test selection radio buttons. Display description of
    selected test and enable RUN TEST button.
    """
    rb = radiovar.get()

    descr.config(state='normal')
    descr.delete('1.0', Tk.END)
    descr.insert(Tk.END, testdescrs[rb][0], 'htag')
    descr.tag_config('htag', font=tabbold, justify='center')
    descr.insert(Tk.END, '\n\n' + testdescrs[rb][1])
    descr.config(state='disabled')

    runbutton.config(state='normal')
    runbutton.config(background='LightBlue2')

def buttonblink():
    """Pulse the background of the RUN TEST button while test is running.
    """
    global rbcolour, blink
    blink = None
    runbutton.config(background=rbcolour)
    if rbcolour == 'pink':
        rbcolour = runbutton.cget('disabledforeground')
    else:
        rbcolour = 'pink'
    blink = root.after(900, buttonblink)

def runtest():
    """Run the selected test after disabling the RUN TEST button and starting
    it pulsing.
    """
    global rbcolour, blink

    rb = radiovar.get()

    runbutton.config(state='disabled')
    rbcolour = 'pink'
    buttonblink()
    tests[rb]()
    root.after_cancel(blink)
    runbutton.config(state='normal')
    runbutton.config(background='LightBlue2')

def openControlStreams():
    global HOST3, DESCR3, ctrl1, ctrl2, ctrl3

    # open the control streams that these tests need
    # (ADI-TTCP   MUST   already be running on the hosts)

    # first see if a RELAY host has been specified on the command line
    if len(sys.argv) > 1:
        HOST3 = sys.argv[1]
        if len(sys.argv) > 2:
            DESCR3 = sys.argv[2]
        ctrl3 = ttcp.CtrlStream(host=HOST3, descr=DESCR3)
    else:
        ctrl3 = None

    # now open the CtrlStreams for the actual test machines
    ctrl1 = ttcp.CtrlStream(host=HOST1, descr=DESCR1, relay=ctrl3)
    ctrl2 = ttcp.CtrlStream(host=HOST2, descr=DESCR2, relay=ctrl3)

def runTests():
    """Loop forever running the tests in sequence."""
    while True:
        for rb in [rb1, rb2, rb3]:
            rb.select()
            rb.invoke()
            runbutton.invoke()
            root.update_idletasks()
            time.sleep(3)

################################################################################

# create main top-level window and some fonts
root = Tk.Tk()
root.title("ADI-TTCP Control and Results")
root.resizable(1,1)
root.geometry("-10+10")
root.protocol('WM_DELETE_WINDOW', program_exit)

tabfont   = Tf.Font(root, ('Courier New', 8, Tf.NORMAL))
tabbold   = Tf.Font(root, ('Arial', 8, Tf.BOLD))
tabcap    = Tf.Font(root, ('Arial', 12, Tf.BOLD))
descrfont = Tf.Font(root, ('Arial', 8, Tf.NORMAL))

# create a frame for the result tables and the tables (and their arrays)
tframe = Tk.Frame(root)
tab1, arr1 = create_table(tframe, 'TCP WRITE', 0, 0)
tab2, arr2 = create_table(tframe, 'TCP READ',  0, 1)
tab3, arr3 = create_table(tframe, 'UDP WRITE', 1, 0)
tab4, arr4 = create_table(tframe, 'UDP READ',  1, 1)

tframe.pack(side='top', expand=1, fill='both')

# create overall frame for lower part of window (below the tables)
cframe = Tk.Frame(root, padx=3, pady=3, relief='groove', borderwidth=2)

# create frame for radio buttons and the buttons themselves
rframe = Tk.Frame(cframe, padx=3, pady=3, relief='groove', borderwidth=2)

radiovar = Tk.IntVar(root)
radiovar.set(-1)

rb1 = Tk.Radiobutton(rframe,
                     variable=radiovar,
                     value=0,
                     anchor='w',
                     text='Test peak throughput versus buffer size',
                     command=radiocheck)
rb1.grid(sticky='ew', column=0, row=0)
rb2 = Tk.Radiobutton(rframe,
                     variable=radiovar,
                     value=1,
                     anchor='w',
                     text='Test CPU use versus data rate',
                     command=radiocheck)
rb2.grid(sticky='ew', column=0, row=1)
rb3 = Tk.Radiobutton(rframe,
                     variable=radiovar,
                     value=2,
                     anchor='w',
                     text='Test CPU use versus number of streams',
                     command=radiocheck)
rb3.grid(sticky='ew', column=0, row=2)

rframe.pack(side='left', expand=1, fill='both')

# create a scrolling text window for the test descriptions
descr = St.ScrolledText(cframe,
                        state='disabled',
                        borderwidth=2,
                        relief='groove',
                        font=descrfont,
                        background='light yellow',
                        wrap='word',
                        height=6,
                        width=45)
descr.pack(side='left', expand=1, fill='both')

# insert initial helpful message in text window
descr.config(state='normal')
descr.insert(Tk.END,
"""Choose a test to see its description.

Press the RUN TEST button to run the selected test and see its results.
"""
            )
descr.config(state='disabled')

# create the RUN TEST button, initially disabled
runbutton = Tk.Button(cframe,
                      state='disabled',
                      text='RUN TEST',
                      foreground='black',
                      background='white',
                      relief='raised',
                      borderwidth=2,
                      command=runtest,
                      font=tabcap)
runbutton.pack(side='right', expand=1, fill='both')

cframe.pack(side='bottom', expand=1, fill='both')

# initialise the CtrlStream module and its logging window
ttcp.CtrlStream.init(program_exit)

# schedule function to open control streams
root.after(500, openControlStreams)

# schedule a function to loop forever running the tests
root.after(3000, runTests)

# start the GUI!
Tk.mainloop()
